use std::rc::Rc;

use super::{Environment, Node, NodeEnum, Precedence};

#[derive(Clone, Debug, PartialEq, PartialOrd)]
pub enum Bool {
    True,
    False,
}

#[derive(Clone, Debug, PartialEq, PartialOrd)]
pub enum ElseBranchEnum {
    ElseIf(Rc<IfElse>),
    Block(Vec<Rc<NodeEnum>>),
}

#[derive(Clone, Debug, PartialEq, PartialOrd)]
pub struct IfElse {
    condition: Rc<NodeEnum>,
    true_branch: Vec<Rc<NodeEnum>>,
    else_branch: ElseBranchEnum,
}

impl Node for IfElse {
    fn evaluate(&self, env: &mut Environment) -> Result<Rc<NodeEnum>, String> {
        let condition_evaluated = self.condition.evaluate(env)?;

        if let super::NodeEnum::Bool(bool) = condition_evaluated {
        } else {
            return Err(format!(
                "Cannot evaluate {} to a bool",
                condition_evaluated.as_string(Some(env))
            ));
        }
    }

    fn as_string(&self, env: Option<&Environment>) -> String {
        format!(
            "if {} then\n{}\nelse\n{}",
            self.condition.as_string(env),
            self.true_branch
                .iter()
                .map(|x| x.as_string(env))
                .reduce(|a, b| a + "\n" + &b)
                .unwrap()
                .as_str(),
            match &self.else_branch {
                ElseBranchEnum::ElseIf(if_else) => if_else.as_string(env),
                ElseBranchEnum::Block(vec) =>
                    vec.iter()
                        .map(|x| x.as_string(env))
                        .reduce(|a, b| a + "\n" + &b)
                        .unwrap()
                        + "\nend",
            }
            .as_str()
        )
    }

    fn precedence(&self) -> Precedence {
        Precedence::Primary
    }
}
